import java.io.BufferedOutputStream;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.IOException;
import java.net.Socket;
import java.net.UnknownHostException;

public class MessageSender {

  Socket sock;

  BufferedOutputStream outStream;

  BufferedReader consoleIn;

  MessageSender(String host, int port) 
    throws IOException, UnknownHostException {

    sock = new Socket(host, port);

    outStream = new BufferedOutputStream(sock.getOutputStream());
    byte[] buf = new byte[1024];
    
    consoleIn = new BufferedReader(new InputStreamReader(System.in));
  }

  public void sendMessage() throws IOException {
    System.out.print(">");
    System.out.flush();
    String message;
    message = consoleIn.readLine();
    outStream.write(message.getBytes(),0,message.length());
    outStream.flush();
    outStream.close();
  }

  public void shutDown() {
    try {
      sock.close();
    }
    catch (IOException e) {
      System.out.println("error closing socket");
      System.exit(0);
    }
  }

  public static void terminate(String errorMessage) {
    System.out.println(errorMessage);
    System.exit(0);
  }

  public static void main(String args[]) {
    MessageSender myMessageSender = null;
    int port = -1;
    if (args.length < 2)
      terminate("host and port required as arguments");

    try {
      port = Integer.parseInt(args[1]);
    }
    catch (NumberFormatException e) {
      terminate("port invalid");
    }
    try {
      myMessageSender = new MessageSender(args[0],port);
    }
    catch (UnknownHostException e) {
      terminate("UnknownHostException");
    }
    catch (IOException e) {
      System.out.println(e.getMessage());
      terminate("IOException");
    }

    try {
      myMessageSender.sendMessage();
    }
    catch (IOException e) {
      terminate("send failed");
    }
    myMessageSender.shutDown();
  }
}
