// WavePlayer.cpp

#include "stdafx.h"
#include <stdexcept>
#include "WavePlayer.h"

namespace DDJCColumnWave	{

WavePlayer::WavePlayer(int b, int rate, int nc) : Wave(b, rate, nc, 4096)
{
	stopping = false;
	playing = false;
	paused = false;
	wavedevice = 0;
	starting = true;
	hdr1.dwFlags = 0;
	hdr2.dwFlags = 0;
	if (waveOutOpen(&wavedevice, WAVE_MAPPER, &wfmt, 
		(DWORD)waveProc, (DWORD)this, CALLBACK_FUNCTION) == 0)	{
		if (waveOutPrepareHeader(wavedevice, &hdr1, sizeof(WAVEHDR)) == 0)
			if (waveOutPrepareHeader(wavedevice, &hdr2, sizeof(WAVEHDR)) == 0)
				return;
	}
	Shutdown();
	throw std::runtime_error("Error initializing playback device");
}
WavePlayer::~WavePlayer()
{
	Shutdown();
}
void WavePlayer::Shutdown()
{
	waveOutUnprepareHeader(wavedevice, &hdr1, sizeof(WAVEHDR));
	waveOutUnprepareHeader(wavedevice, &hdr2, sizeof(WAVEHDR));
	do
		Sleep(50);
	while (waveOutClose(wavedevice) == WAVERR_STILLPLAYING);
}

void WavePlayer::Play()
{
	if (paused)
		Resume();
	else	{
		playing = true;
		starting = true;
		FillBufferAndPlay(&hdr1);
		FillBufferAndPlay(&hdr2);
		starting = false;
		AfxBeginThread(&PlayBuffers, this);
	}
}

UINT WavePlayer::PlayBuffers(void* p)
{
	WavePlayer& wp = *(static_cast<WavePlayer*>(p));
	bool done = false;
	while (!done)	{
		DWORD ev = WaitForMultipleObjects(3, wp.audioevents, FALSE, INFINITE);
		switch (ev - WAIT_OBJECT_0)	{
			case process1:
				wp.FillBufferAndPlay(&wp.hdr1);
				break;
			case process2:
				wp.FillBufferAndPlay(&wp.hdr2);
				break;
			case close:
				done = true;
				break;
			default:
				break;
		}
	}
	return 0;
}

void WavePlayer::Stop()
{
	stopping = playing;
	if (paused)
		Resume();
}

void WavePlayer::Pause()
{
	if (!paused)	{
		paused = true;
		waveOutPause(wavedevice);
	}
}

void WavePlayer::Resume()
{
	if (paused)	{
		paused = false;
		waveOutRestart(wavedevice);
	}
}

void WavePlayer::FillBufferAndPlay(PWAVEHDR hdr)
{
	if (playing)	{
		if (!stopping && !paused)	{
			if (starting || hdr->dwFlags & WHDR_DONE)	{
				int len;
				len = FillBuffer(hdr->lpData, buflen); // passes a byte count
				if (len > 0)
					len = DSP(hdr->lpData, len);
				if (len > buflen * 2)
					throw std::logic_error("FillBuffer/DSP returns too many samples");
				if (len > 0)	{
					hdr->dwBufferLength = len;
					if (waveOutWrite(wavedevice, hdr, sizeof(WAVEHDR)) != 0)
						throw std::runtime_error("Audio playback error");
				}
			}
		}
		else	{
			while((hdr1.dwFlags & WHDR_DONE) == 0 && 
					(hdr2.dwFlags & WHDR_DONE) == 0)
				Sleep(50);
			if (stopping)	{
				playing = false;
				paused = false;
			}
		}
	}
}

} // namespace DDJCColumnWave
