// WaveRecorder.cpp
#include "stdafx.h"
#include <stdexcept>
#include "WaveRecorder.h"

namespace DDJCColumnWave	{

WaveRecorder::WaveRecorder(int b, int rate, int nc) : Wave(b, rate, nc, 1024)
{
	wavedevice = 0;
	stopping = false;
	if (waveInOpen(&wavedevice, WAVE_MAPPER, &wfmt, 
			(DWORD)waveProc, (DWORD)this, CALLBACK_FUNCTION) == 0)
		if (waveInPrepareHeader(wavedevice, &hdr1, sizeof(WAVEHDR)) == 0)
			if (waveInPrepareHeader(wavedevice, &hdr2, sizeof(WAVEHDR)) == 0)
				return;
	ShutDown();
	throw std::runtime_error("Error initializing recorder device");
}
WaveRecorder::~WaveRecorder()
{
	ShutDown();
}
void WaveRecorder::ShutDown()
{
	stopping = true;
	waveInStop(wavedevice);
	waveInReset(wavedevice);
	waveInUnprepareHeader(wavedevice, &hdr1, sizeof(WAVEHDR));
	waveInUnprepareHeader(wavedevice, &hdr2, sizeof(WAVEHDR));
	waveInClose(wavedevice);
}
void WaveRecorder::Record()
{
	waveInAddBuffer(wavedevice, &hdr1, sizeof(WAVEHDR));
	waveInAddBuffer(wavedevice, &hdr2, sizeof(WAVEHDR));
	waveInStart(wavedevice);
	AfxBeginThread(&RecordBuffers, this);
}
UINT WaveRecorder::RecordBuffers(void* p)
{
	TRACE("\n===In Thread");
	WaveRecorder& wr = *(static_cast<WaveRecorder*>(p));
	bool done = false;
	while (!done)	{
		DWORD ev = WaitForMultipleObjects(3, wr.audioevents, FALSE, INFINITE);
		switch (ev - WAIT_OBJECT_0)	{
			case process1:
				wr.SaveBuffer(&wr.hdr1);
				break;
			case process2:
				wr.SaveBuffer(&wr.hdr2);
				break;
			case close:
				done = true;
				break;
			default:
				break;
		}
	}
	return 0;
}
void WaveRecorder::SaveBuffer(PWAVEHDR hdr)
{
	if (!stopping)	{
		StoreData(hdr->lpData, hdr->dwBytesRecorded);
		waveInAddBuffer(wavedevice, hdr, sizeof(WAVEHDR));
	}
}

} // namespace DDJCColumnWave
