package DE.FhG.IGD.crypto;
import java.security.*;

/**
 * A Java Security Provider according to the JCE specification.
 * Provides basic cryptographic services, in particular those
 * which seem broken in other providers such as:
 * <ul>
 * <li> AlgorithmParameters
 * <li> AlgorithmParameterGenerators
 * </ul>
 *
 * This provider furthermore defines a number of aliases that
 * are ususally not found in other providers, but which are
 * neccessary for such providers to work with tools such as
 * the keytool and jarsigner.
 *
 * @author Volker Roth
 * @version "$Id: A8Provider.java,v 1.3 2001/03/23 15:59:46 vroth Exp $"
 * @see A8KeyStore
 */
public class A8Provider extends Provider
{
    /**
     * The name of this provider.
     */
    public static final String name = "A8";


    /**
     * Creates the A8Provider. This provider constructor registers the
     * keystores, ciphers, key factories etc. supported by it.
     */
    public A8Provider()
	{
            super(name, 1.2, "A8 Basic Java Crypto Services");
            AccessController.doPrivileged(new Defines());
	}

    private class Defines extends Object implements PrivilegedAction
     {
         public Object run()
            {
                /* Message Digest Algorithm OID
                 * SHA, MD5, MD4, MD2
                 */
                put("Alg.Alias.MessageDigest.OID.1.3.14.3.2.26",
                    "SHA");
                put("Alg.Alias.MessageDigest.1.3.14.3.2.26",
                    "SHA");
                put("Alg.Alias.MessageDigest.OID.1.2.840.113549.2.5",
                    "MD5");
                put("Alg.Alias.MessageDigest.1.2.840.113549.2.5",
                    "MD5");
                put("Alg.Alias.MessageDigest.OID.1.2.840.113549.2.4",
                    "MD4");
                put("Alg.Alias.MessageDigest.1.2.840.113549.2.4",
                    "MD4");
                put("Alg.Alias.MessageDigest.OID.1.2.840.113549.2.2",
                    "MD2");
                put("Alg.Alias.MessageDigest.1.2.840.113549.2.2",
                    "MD2");

                /* Signature engines
                 */
                put("Alg.Alias.Signature.MD5/RSA",
                    "MD5withRSA");
                put("Alg.Alias.Signature.1.2.840.113549.1.1.4",
                    "MD5withRSA");
                put("Alg.Alias.Signature.1.3.14.3.2.25",
                    "MD5withRSA");
                put("Alg.Alias.Signature.OID.1.2.840.113549.1.1.4",
                    "MD5withRSA");

                /* Cipher engines
                 */
                put("Alg.Alias.Cipher.OID.1.2.840.113549.3.7",
                    "DESede/CBC/PKCS5Padding");
                put("Alg.Alias.Cipher.1.2.840.113549.3.7",
                    "DESede/CBC/PKCS5Padding");

                put("Alg.Alias.Cipher.1.2.840.113549.3.2",
                    "RC2/CBC/PKCS5Padding");
                put("Alg.Alias.Cipher.OID.1.2.840.113549.3.2",
                    "RC2/CBC/PKCS5Padding");
                
                put("Alg.Alias.Cipher.OID.1.2.840.113549.1.5.1",
                    "PBEWithMD2AndDES");
                put("Alg.Alias.Cipher.1.2.840.113549.1.5.1",
                    "PBEWithMD2AndDES");
                
                put("Alg.Alias.Cipher.OID.1.2.840.113549.1.5.3",
                    "PBEWithMD5AndDES");
                put("Alg.Alias.Cipher.1.2.840.113549.1.5.3",
                    "PBEWithMD5AndDES");
                
                put("Alg.Alias.Cipher.OID.1.2.840.113549.1.5.12",
                    "PBEWithSHA1And128BitRC4");
                put("Alg.Alias.Cipher.1.2.840.113549.1.5.12",
                    "PBEWithSHA1And128BitRC4");

                put("Alg.Alias.Cipher.OID.1.2.840.113549.1.1.1",
                    "RSA");
                put("Alg.Alias.Cipher.1.2.840.113549.1.1.1",
                    "RSA");

                /* (Secret)KeyFactories
                 */
                put("Alg.Alias.SecretKeyFactory.1.2.840.113549.3.7",
                    "DESede/CBC/PKCS5Padding");
                put("Alg.Alias.KeyFactory.1.2.840.113549.1.1.1",
                    "RSA");

                /* AlgorithmParameters
                 */
                put("Alg.Alias.AlgorithmParameters.1.2.840.113549.3.7",
                    "DESede");
                put("Alg.Alias.AlgorithmParameters.DESede/CBC/PKCS5Padding",
                    "DESede");
                
                put("Alg.Alias.AlgorithmParameters.1.2.840.113549.3.2",
                    "RC2");
                put("AlgorithmParameters.RC2",
                    "DE.FhG.IGD.crypto.RC2Parameters");
                put("AlgorithmParameterGenerator.RC2",
                    "DE.FhG.IGD.crypto.RC2ParameterGenerator");
                
                put("Alg.Alias.AlgorithmParameters.1.2.840.113549.1.5.1",
                    "PBE");// PBEWithMD2AndDES
                put("Alg.Alias.AlgorithmParameters.1.2.840.113549.1.5.3",
                    "PBE");// PBEWithMD5AndDES
                put("Alg.Alias.AlgorithmParameters.1.2.840.113549.1.5.12",
                    "PBE");// PBEWithSHA1And128BitRC4
                
                put("Alg.Alias.AlgorithmParameters.PBEWithMD2AndDES",
                    "PBE");// PBEWithMD2AndDES
                put("Alg.Alias.AlgorithmParameters.PBEWithMD5AndDES",
                    "PBE");// PBEWithMD5AndDES
                put("Alg.Alias.AlgorithmParameters.PBEWithSHA1And128BitRC4",
                    "PBE");// PBEWithSHA1And128BitRC4
                
                put("AlgorithmParameters.DESede",
                    "DE.FhG.IGD.crypto.DESParameters");
                put("AlgorithmParameterGenerator.DESede",
                    "DE.FhG.IGD.crypto.DESParameterGenerator");

                put("AlgorithmParameters.PBE",
                    "DE.FhG.IGD.crypto.PBEParameters");
                put("AlgorithmParameterGenerator.PBE",
                    "DE.FhG.IGD.crypto.PBEParameterGenerator");
                
                put("AlgorithmParameterGenerator.PBEWithMD2AndDES",
                    "DE.FhG.IGD.crypto.PBEParameterGenerator");
                
                put("AlgorithmParameterGenerator.PBEWithMD5AndDES",
                    "DE.FhG.IGD.crypto.PBEParameterGenerator");
                
                put("AlgorithmParameterGenerator.PBEWithSHA1And128BitRC4",
                    "DE.FhG.IGD.crypto.PBEParameterGenerator");
                
                put("Alg.Alias.AlgorithmParameters.1.3.14.3.2.12",
                    "DSA");
                put("Alg.Alias.AlgorithmParameters.1.2.840.10040.4.1",
                    "DSA");
                
                return null;
            }
     }


}
