#  $Id: GUIFactory.py,v 1.14.1.7.1.4 2001-08-14 14:40:09-05 brinkman Exp $
#  A class that provides high level GUI features.
#  Copyright (C) 2000  Peter Brinkmann (brinkman@math.uiuc.edu)
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

"""
GUIFactory.py: Provides the default widget factory for awt (Java 1.1).

$Id: GUIFactory.py,v 1.14.1.7.1.4 2001-08-14 14:40:09-05 brinkman Exp $
"""


import java
import Config
import ImageCanvas
import string

if 0==1:	# this funny-looking hack forces jythonc to create some
			# adapters that it would otherwise miss.
	java.awt.CheckboxMenuItem.itemStateChanged
	java.awt.Button.actionPerformed
	java.awt.Button.keyTyped
	java.awt.TextArea.textValueChanged

###############################################################################

class Factory:
	"""
This is the default factory that constructs awt widgets. Pretty
straightforward altogether.
"""

	defaultfont=java.awt.Font('Helvetica',java.awt.Font.BOLD,16)
	dialogfont=java.awt.Font('Helvetica',java.awt.Font.BOLD,18)
	filedialogfont=defaultfont
	textfont=java.awt.Font('Helvetica',java.awt.Font.PLAIN,12)
	textfontlarge=java.awt.Font('Helvetica',java.awt.Font.PLAIN,14)

	dialogbackcolor=None
	dialogtextcolor=None

	LOAD=java.awt.FileDialog.LOAD
	SAVE=java.awt.FileDialog.SAVE

	locations={     'N':java.awt.BorderLayout.NORTH,
			'E':java.awt.BorderLayout.EAST,
			'W':java.awt.BorderLayout.WEST,
			'S':java.awt.BorderLayout.SOUTH,
			'C':java.awt.BorderLayout.CENTER
		}

	isMac=(string.find(java.lang.System.getProperty('os.name'),
								'Mac')>=0)

	frameClass=java.awt.Frame

	def constructFrame(self,*args,**kwargs):
		return apply(self.frameClass,args,kwargs)

	def constructMenuBar(self,*args,**kwargs):
		return apply(java.awt.MenuBar,args,kwargs)

	def constructPopupMenu(self,*args,**kwargs):
		return apply(java.awt.PopupMenu,args,kwargs)

	def constructMenu(self,*args,**kwargs):
		return apply(java.awt.Menu,args,kwargs)

	def constructCheckboxMenuItem(self,*args,**kwargs):
		return apply(java.awt.CheckboxMenuItem,args,kwargs)

	def constructMenuItem(self,*args,**kwargs):
		return apply(java.awt.MenuItem,args,kwargs)

	def constructMenuShortcut(self,*args,**kwargs):
		return apply(java.awt.MenuShortcut,args,kwargs)

	def constructButton(self,*args,**kwargs):
		return apply(java.awt.Button,args,kwargs)

	def constructLabel(self,*args,**kwargs):
		return apply(java.awt.Label,args,kwargs)

	def constructTextArea(self,*args,**kwargs):
		return apply(java.awt.TextArea,args,kwargs)

	def constructPanel(self,*args,**kwargs):
		return apply(java.awt.Panel,args,kwargs)

	def constructBorderLayout(self,*args,**kwargs):
		return apply(java.awt.BorderLayout,args,kwargs)

	def constructGridLayout(self,*args,**kwargs):
		return apply(java.awt.GridLayout,args,kwargs)

	def constructFlowLayout(self,*args,**kwargs):
		return apply(java.awt.FlowLayout,args,kwargs)

	def constructCardLayout(self,*args,**kwargs):
		return apply(java.awt.CardLayout,args,kwargs)

	def constructImageCanvas(self,*args,**kwargs):
		return apply(ImageCanvas.ImageCanvas,args,kwargs)

	def constructQDialog(self,*args,**kwargs):
		return apply(QDialog,args,kwargs)

	def constructTDialog(self,*args,**kwargs):
		return apply(TDialog,args,kwargs)

	def loadDialog(self,master,title,func):
		return self._fileDialog(master,title,func,0)

	def saveDialog(self,master,title,func):
		return self._fileDialog(master,title,func,1)

	def constructWaitCursor(self):
		return java.awt.Cursor(java.awt.Cursor.WAIT_CURSOR)

	def _fileDialog(self,master,title,func,mode=0):
		if not master.isApplet:
			if mode==0:
				fd=FDialog(master,master.frame,title,
					self.LOAD)
			else:
				fd=FDialog(master,master.frame,title,
					self.SAVE)
			fd.func=func
			fd.setFont(self.filedialogfont)
			fd.show()
			fn=fd.getFile()
			di=fd.getDirectory()
			if fn:
				master.recent_dir=di
				if java.io.File(di,fn).exists():
					if mode!=0:
						YES=_('Yes')
						ans=YES
						if not self.isMac:
							ans=master.question(master.name,
								fn+_(': File exists. Overwrite?'),
								((YES,
								lambda:None),
							 	(_('No'),
								lambda:None)))
						if ans==YES:
							fd.doit()
							fd.hide()
							return (di,fn)
					else:
						fd.doit()
						fd.hide()
						return (di,fn)
				else:
					if mode!=0:
						fd.doit()
						fd.hide()
						return (di,fn)
					else:
						master.errorBox(fn+_(': No such file.'))
		fd.hide()
		return None


###############################################################################


class QDialog(java.awt.Dialog):
	"""
A subclass of Dialog that provides some extra functionality. Typically,
instances of this class will only be created by MenuMaker.
"""

	def __init__(self,ms,fr,tl,text,opts,mod):
		"""
constructor for QDialog.
Parameters:
ms: the master object that created this instance
fr: the frame this instance is attached to
tl: title string
text: text of the label. If text contains the characted '\n', then
      text will be split at occurrences of '\n', and each substring
      will appear in a separate line.
opts: a list of buttons, of the form
	 [(label1,function1),(label2,function2),...],
      where the labels are the labels of buttons, and a function
      will be called when the user clicks on the corresponding button.
mod: a flag indicating whether this dialog is modal
Moreover, the master object can set the font of the dialog by changing the
value of the variable self.dialogfont.
"""
		self.reply=None
		java.awt.Dialog.__init__(self,fr,tl,mod)
		self.windowClosing=self.close
		self.setBackground(Config.factory.dialogbackcolor)
		self.setForeground(Config.factory.dialogtextcolor)
		self.contents=None
		self.fr=fr
		self.setFont(Config.factory.dialogfont)
		self.setLabel(text)
		if len(opts)>0:
			self.makeButtons(opts)


	def makeButtons(self,opts):
		p=java.awt.Panel()
		if len(opts)>3:
			s=0
			while (s*s<len(opts)):
				s=s+1
			p.setLayout(java.awt.GridLayout(s,s))
		self.options={}
		for i in opts:
			self.options[i[0]]=i[1]
			p.add(java.awt.Button(i[0],
				actionPerformed=self.evalButton))
		self.add(p,java.awt.BorderLayout.SOUTH)


	def setLabel(self,text):
		"""
Sets the label of this instance.
Parameters:
text: the new text of the label.
"""
		try:
			self.remove(self.contents)
		except:
			pass
		ll=string.split(text,'\n')
		if len(ll)<2:
			self.contents=java.awt.Label(text)
			self.add(self.contents,
				java.awt.BorderLayout.CENTER)
		else:
			p=java.awt.Panel()
			p.setLayout(java.awt.GridLayout(len(ll),1))
			for i in ll:
				p.add(java.awt.Label(i))
			self.add(p,java.awt.BorderLayout.CENTER)
			self.contents=p
		self.pack()


	def show(self):
		"""
Displays this dialog at the center of the ambient frame (more or less...)
"""
		self.pack()
		if self.fr.isVisible():
			p=self.fr.getLocationOnScreen()
			d=self.fr.getSize()
			cx=(p.x+d.width)/2
			cy=(p.y+d.height)/2
			d=self.getSize()
			x=cx-d.width/2
			y=cy-d.height/2
			self.setLocation(x,y)
		java.awt.Dialog.show(self)


	def evalButton(self,ev):
		self.reply=ev.getActionCommand()
		if self.__dict__.has_key('options'):
			self.options[self.reply]()
		self.close()


	def close(self,ev=None):
		"""
Closes this dialog and prepares it for garbage collection.
"""
		self.hide()


###############################################################################


class FDialog(java.awt.FileDialog):
	"""
A subclass of FileDialog, only to be used from within MenuMaker.
"""
	def __init__(self,owner,fr,title,mode):
		java.awt.FileDialog.__init__(self,fr,title,mode)
		self.windowClosing=self.close
		self.setBackground(Config.factory.dialogbackcolor)
		self.setForeground(Config.factory.dialogtextcolor)
		if owner.__dict__.has_key('recent_dir'):
			self.setDirectory(owner.recent_dir)

	def doit(self):
		if self.getFile():
			try:
				self.parent.setAllCursorsBusy()
				self.func(
				   self.getDirectory()+self.getFile())
			finally:
				self.parent.resetAllCursors()

	def close(self):
		self.hide()


###############################################################################

class TDialog(java.awt.Dialog):
	"""
A dialog class for entering strings.
Note that morally, it would make sense to derive this class from
QDialog, but a bug in the current version of jython (version 2.0)
makes this impossible.	08/11/2001
"""

	def __init__(self,ms,fr,title,text,value,columns,subm,
			canc,echo):
		java.awt.Dialog.__init__(self,fr,title,1)
		self.windowClosing=self.close
		self.fr=fr
		self.value=None
		self.setLayout(java.awt.FlowLayout())
		self.setBackground(Config.factory.dialogbackcolor)
		self.setForeground(Config.factory.dialogtextcolor)
		self.setFont(Config.factory.dialogfont)
		self.add(java.awt.Label(text))
		if value!=None:
			if columns!=None:
				self.tf=java.awt.TextField(
						value,columns)
			else:
				self.tf=java.awt.TextField(
						value,len(value)+5)
			self.tf.select(0,len(value))
		else:
			if columns!=None:
				self.tf=java.awt.TextField("",columns)
			else:
				self.tf=java.awt.TextField("",20)
		if echo!=None:
			self.tf.setEchoChar(echo)
		self.tf.keyTyped=self.evalKey
		self.add(self.tf)
		if subm!=None:
			self.add(java.awt.Button(subm,
				actionPerformed=self.submit))
		if canc!=None:
			self.add(java.awt.Button(canc,
				actionPerformed=self.close))

	def evalKey(self,ev=None):
		if ord(ev.keyChar)==10:
			self.submit()

	def submit(self,ev=None):
		self.value=self.tf.getText()
		self.close()

	def show(self):
		"""
Displays this dialog at the center of the ambient frame (more or less...)
"""
		self.pack()
		if self.fr.isVisible():
				p=self.fr.getLocationOnScreen()
				d=self.fr.getSize()
				cx=(p.x+d.width)/2
				cy=(p.y+d.height)/2
				d=self.getSize()
				x=cx-d.width/2
				y=cy-d.height/2
				self.setLocation(x,y)
		java.awt.Dialog.show(self)


	def close(self,ev=None):
		"""
Closes this dialog and prepares it for garbage collection.
"""
		self.hide()

###############################################################################

