#  $Id: ImageCanvas.py,v 2.2 2001-02-23 19:18:44-06 brinkman Exp $
#  A subclass of java.awt.Canvas that displays an image.
#  Copyright (C) 2000  Peter Brinkmann (brinkman@math.uiuc.edu)
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

"""
ImageCanvas.py: An extension of java.awt.Canves for drawing images.

$Id: ImageCanvas.py,v 2.2 2001-02-23 19:18:44-06 brinkman Exp $
"""

import java


###############################################################################

class ImageCanvas(java.awt.Canvas):
	"""
ImageCanvas: An extension of java.awt.Canvas that contains an image (self.img)
to be drawn whenever this canvas is painted.
"""

###############################################################################

	def fromFile(self,fn):
		"""
Reads an image from the file named fn.

Parameters:

	- fn: file name
"""
		self.setImage(
			java.awt.Toolkit.getDefaultToolkit().getImage(fn))

###############################################################################

	def fromString(self,imgstr):
		"""
Reads an image from a given string. The idea is to allow developers to encode
images in python strings rather than files, which makes it easier to locate and
load images.

Parameters:

	- imgstr: string containing image
"""
		self.setImage(
			java.awt.Toolkit.getDefaultToolkit().createImage(
			java.lang.String(imgstr).getBytes()))

###############################################################################

	def setImage(self,img):
		"""
Sets self.img to the given image img, and sets the size of this instance to be
equal to the size of the image.

Parameters:

	- img: image
"""
		self.img=img
		self.setPrefSize()

###############################################################################

	def prefWidth(self):
		"""
Returns the width of self.img.
"""
		self._waitForImage()
		return self.img.getWidth(self)

###############################################################################

	def prefHeight(self):
		"""
Returns the height of self.img.
"""
		self._waitForImage()
		return self.img.getHeight(self)

###############################################################################

	def setPrefSize(self):
		"""
Sets the size of this instance to the size of self.img (plus 20 points for
margins).
"""
		self.setSize(self.prefWidth()+20,self.prefHeight()+20)

###############################################################################
# The remainder is for internal use only.                                     #
###############################################################################

	def _waitForImage(self):
		while (self.img.getWidth(self)<0 or
				 self.img.getHeight(self)<0):
			pass

	def paint(self,gr):
		gr.drawImage(self.img,10,10,self)

###############################################################################


