#  $Id: PowerFrame2.py,v 1.4 2001-08-16 15:34:26-05 brinkman Exp $
#  A frame with high level GUI features inherited from MenuMaker.
#  Copyright (C) 2000  Peter Brinkmann (brinkman@math.uiuc.edu)
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

"""
PowerFrame.py: A class that inherits both from MenuMaker and some frame class.

$Id: PowerFrame2.py,v 1.4 2001-08-16 15:34:26-05 brinkman Exp $
"""

import Config
from MenuMaker import MenuMaker


###############################################################################

class PowerFrame(MenuMaker,Config.factory.frameClass):
	"""
$Id: PowerFrame2.py,v 1.4 2001-08-16 15:34:26-05 brinkman Exp $

A frame that extends MenuMaker. By default, all the widgets created by
MenuMaker are supposed to live in the northern, southern, eastern, or
western part of the frame's layout. The center is reserved for widgets
of the subclass of PowerFrame (this behavior can be changed, though!).
The class PowerFrame maintains a static list of all frames belonging to
subclasses of PowerFrame.
"""

	version="""
$Id: PowerFrame2.py,v 1.4 2001-08-16 15:34:26-05 brinkman Exp $
(c) 2000 Peter Brinkmann (brinkman@math.uiuc.edu)
"""
	helptext="""
Override this with help for subclasses of PowerFrame!
"""
	frameList=[]

###############################################################################

	def __init__(self,ms,name='PowerFrame'):
		"""
Constructor for PowerFrame.

Parameters:

	- ms: Either None or an object that owns this instance.

"""
		MenuMaker.__init__(self,ms,1,name)
		PowerFrame.frameList.append(self)
		self.windowClosing=self.close
		self.setFont(Config.factory.defaultfont)
		self.setTitle(name)
		if self.menubar!=None:
			self.setMenuBar(self.menubar)
		for i in self.toolbars.keys():
			self.add(self.toolbars[i],Config.factory.locations[i])
		self.setLocation(50,50)

###############################################################################

	def close(self,e=None):
		"""
Closes this frame, calls the close method of MenuMaker first and only
proceeds if the return value of MenuMaker.close is nonzero.
"""
		self.toFront()
		if MenuMaker.close(self):
			if self in PowerFrame.frameList:
				PowerFrame.frameList.remove(self)
			self.setVisible(0)
			if len(PowerFrame.frameList)==0 and not MenuMaker.isApplet:
				import sys
				sys.exit(0)
			return 1
		else:
			return 0

###############################################################################

	def exit(self,e=None):
		"""
Exits altogether.
"""
		fl=PowerFrame.frameList[:]
		for i in fl:
			if i in PowerFrame.frameList:
				i.close()

###############################################################################

	def showText(self,ms,title,txt):
		"""
Opens a window that displays some text.

Parameters:

	- ms: owner of the new window

	- title: title of the new window

	- txt: text to be displayed
"""
		PlainFrame(ms,title,txt)

###############################################################################

	def showPic(self,ms,title,img):
		"""
Opens a window that displays some image.

Parameters:

	- ms: owner of the new window

	- title: title of the new window

	- img: string containing the image (e.g., in jpeg format)

Note: The image is supposed to sit in a string because like this, image data
can be coded into the jpython code (which has certain advantages). If you
want to display an image file, you can accomplish this following this example:

	f=open('image.jpg','rb')
	img=f.read()
	f.close()
	self.showPic(ms,'some picture',img)
"""
		PicFrame(ms,title,img)

###############################################################################

###############################################################################




###############################################################################

class PlainFrame(PowerFrame):

	"""
$Id: PowerFrame2.py,v 1.4 2001-08-16 15:34:26-05 brinkman Exp $

A simple window with a TextArea (noneditable) and a 'Dismiss' button,
mostly for use from within PowerFrame or MenuMaker.
"""

	def __init__(self,ms,title,txt):
		"""
Constructor for PlainFrame.

Parameters:

	- ms: owner of this frame

	- title: title string

	- txt: text to be displayed
"""
		PowerFrame.__init__(self,ms,title)
		self.text.setText(txt)
		self.pack()
		self.setVisible(1)

	def start(self):
		self.mBar=[(_('File'),[(_('Close'),
			self.close,1,'c')])]
		self.tBar=[]

	def makeWidgets(self):
		self.text=Config.factory.constructTextArea(20,60)
		self.text.setEditable(0)
		self.text.setFont(Config.factory.textfont)
		self.add(self.text,Config.factory.locations['C'])

###############################################################################



###############################################################################

class PicFrame(PowerFrame):

	"""
$Id: PowerFrame2.py,v 1.4 2001-08-16 15:34:26-05 brinkman Exp $

A simple window with an image and a 'Dismiss' button, mostly for use from
within PowerFrame or MenuMaker.
"""

	def __init__(self,ms,title,img):
		"""
Constructor for PicFrame.

Parameters:

	- ms: owner of this instance

	- title: title string

	- img: string containing the image (e.g., in jpeg format).
"""
		self.img=img
		PowerFrame.__init__(self,ms,title)
		self.pack()
		self.setVisible(1)

	def start(self):
		self.mBar=[(_('File'),[(_('Close'),
				self.close,1,'c')])]
		self.tBar=[]

	def makeWidgets(self):
		self.icv=Config.factory.constructImageCanvas()
		self.icv.fromString(self.img)
		self.add(self.icv,Config.factory.locations['C'])

###############################################################################

