#  $Id: jexec.py,v 2.2 2001-02-23 19:33:26-06 brinkman Exp $
#  A simple wrapper for Runtime.getRuntime().exec(...)
#  Copyright (C) 2000  Peter Brinkmann (brinkman@math.uiuc.edu)
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


"""
jexec.py: A wrapper for executing other programs from within jython.

$Id: jexec.py,v 2.2 2001-02-23 19:33:26-06 brinkman Exp $
"""

import java


###############################################################################

class jexec:
	"""
A class that starts a new process and provides access to the i/o streams
of the new process.

Important variables:

	- self.pid: the process id of the new process

	- self.out: an OutputStreamWriter wrapped around the output stream
			of self.pid

	- self.in: a BufferedReader wrapped around the input stream
			of self.pid

	- self.err: a BufferedReader wrapped around the error stream
			of self.pid
"""

	version="""
$Id: jexec.py,v 2.2 2001-02-23 19:33:26-06 brinkman Exp $
(c) 2000 Peter Brinkmann (brinkman@math.uiuc.edu)
"""

###############################################################################

	def __init__(self,cmd):
		"""
Constructor for jexec.

Arguments:

	- cmd: the command to be executed
"""
		self.pid=java.lang.Runtime.getRuntime().exec(cmd)
		self.out=java.io.OutputStreamWriter(
			self.pid.getOutputStream())
		self.in=java.io.BufferedReader(java.io.InputStreamReader(
			self.pid.getInputStream()))
		self.err=java.io.BufferedReader(java.io.InputStreamReader(
			self.pid.getErrorStream()))

###############################################################################

	def readAll(self):
		"""
Reads from self.in until the stream gets closed.

Returns a string containing all the lines that have been
read (separated by '\n').
"""
		return self.readStream(self.in)

###############################################################################

	def readError(self):
		"""
Reads from self.err until the stream gets closed.

Returns a string containing all the lines that have been
read (separated by '\n').
"""
		return self.readStream(self.err)

###############################################################################

	def readStream(self,stream):
		res=''
		while 1:
			line=stream.readLine()
			if line==None:
				break
			res=res+line+'\n'
		return res

###############################################################################


